import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        this._scene = scene;
        
        this._clock = new THREE.Clock();

        this._setupCamera();
        //this._setupLight();
        this._setupModel();
        //this._setupInteractions();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();
        
        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
        //.
    }

    _setupPostprocessing() {
        //.
    }

    _setupModel() {
        const geometry = new THREE.BufferGeometry();
        const material = new THREE.LineBasicMaterial({ vertexColors: true, morphTargets: true });

        const positions = [];
        const colors = [];

        const segments = 10000;
        const radius = 800;
        
        for(let i=0; i<segments; i++) {
            const x = Math.random() * radius - radius/2;
            const y = Math.random() * radius - radius/2;
            const z = Math.random() * radius - radius/2;

            positions.push(x, y, z);
            
            const r = (x / radius) + 0.5;
            const g = (y / radius) + 0.5;
            const b = (z / radius) + 0.5;

            colors.push(r, g, b);
        }

        geometry.setAttribute("position", new THREE.Float32BufferAttribute(positions, 3));
        geometry.setAttribute("color", new THREE.Float32BufferAttribute(colors, 3));

        this._generateMorphTargets(geometry, radius);

        geometry.computeBoundingSphere(); // 필요한가?

        const line = new THREE.Line(geometry, material);
        this._scene.add(line);

        this._t = 0;
        this._line = line;
    }

    _setupHelper() {
        //.
    }

    _generateMorphTargets(geometry, r) {
        const data = [];

        const segments = geometry.attributes.position.array.length / 3;
        for(let i=0; i<segments; i++) {
            const x = Math.random() * r - r / 2;
            const y = Math.random() * r - r / 2;
            const z = Math.random() * r - r / 2;


            data.push(x, y, z);
        }

        const morphTarget = new THREE.Float32BufferAttribute(data, 3);
        morphTarget.name = "target1";

        geometry.morphAttributes.position = [ morphTarget ];
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;

        const params = {
            morphTargets: false
        }

        const gui = new GUI();
        gui.add(params, "morphTargets");
        gui.open();

        this._params = params;
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(27, window.innerWidth / window.innerHeight, 1, 4000);
        camera.position.z = 2750;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        //.
    }

    update() {
        const delta = this._clock.getDelta();
        const time = this._clock.getElapsedTime();

        this._line.rotation.x = time * 0.25;
        this._line.rotation.y = time * 0.5;

        if(this._params.morphTargets) {
            this._t += delta * 0.5;
            this._line.morphTargetInfluences[0] = Math.abs(Math.sin(this._t));
        }

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    new App();
}